@@ -26,12 +26,11 @@ async def get_request_verification_token(session: ClientSession) -> str:
     :return: the verification token
     """
     try:
-        async with session:
-            http_response = await session.get(
-                "https://webauth.southernco.com/account/login"
-            )
-            login_page = await http_response.text()
-            matches = re.findall(r'data-aft="(\S+)"', login_page)
+        http_response = await session.get(
+            "https://webauth.southernco.com/account/login"
+        )
+        login_page = await http_response.text()
+        matches = re.findall(r'data-aft="(\S+)"', login_page)
     except Exception as error:
         raise CantReachSouthernCompany() from error
     if len(matches) < 1:
@@ -87,6 +86,9 @@ class SouthernCompanyAPI:
         Connects to Southern company and gets all accounts
         """
         self._request_token = await get_request_verification_token(self.session)
+        self._request_token_expiry = datetime.datetime.now() + datetime.timedelta(
+            hours=3
+        )
         self._sc = await self._get_sc_web_token()
         self._jwt = await self.get_jwt()
         self._accounts = await self.get_accounts()
@@ -94,6 +96,9 @@ class SouthernCompanyAPI:
     async def authenticate(self) -> bool:
         """Determines if you can authenticate with Southern Company with given login"""
         self._request_token = await get_request_verification_token(self.session)
+        self._request_token_expiry = datetime.datetime.now() + datetime.timedelta(
+            hours=3
+        )
         self._sc = await self._get_sc_web_token()
         return True
 
@@ -113,18 +118,17 @@ class SouthernCompanyAPI:
             "params": {"ReturnUrl": "null"},
         }
 
-        async with self.session as session:
-            async with session.post(
-                "https://webauth.southernco.com/api/login", json=data, headers=headers
-            ) as response:
-                if response.status != 200:
-                    raise CantReachSouthernCompany()
-                try:
-                    connection = await response.json()
-                except (ContentTypeError, json.JSONDecodeError) as err:
-                    raise InvalidLogin from err
-                if connection["statusCode"] == 500:
-                    raise InvalidLogin()
+        async with self.session.post(
+            "https://webauth.southernco.com/api/login", json=data, headers=headers
+        ) as response:
+            if response.status != 200:
+                raise CantReachSouthernCompany()
+            try:
+                connection = await response.json()
+            except (ContentTypeError, json.JSONDecodeError) as err:
+                raise InvalidLogin from err
+            if connection["statusCode"] == 500:
+                raise InvalidLogin()
         sc_regex = re.compile(r"NAME='ScWebToken' value='(\S+.\S+.\S+)'", re.IGNORECASE)
         sc_data = sc_regex.search(connection["data"]["html"])
         self._sc_expiry = datetime.datetime.now() + datetime.timedelta(hours=3)
@@ -134,6 +138,7 @@ class SouthernCompanyAPI:
             else:
                 return sc_data.group(1)
         else:
+            self._sc = None
             raise NoScTokenFound("Login request did not return a sc token")
 
     async def _get_southern_jwt_cookie(self) -> str:
@@ -141,38 +146,38 @@ class SouthernCompanyAPI:
         if await self.sc is None:
             raise CantReachSouthernCompany("Sc token cannot be refreshed")
         data = {"ScWebToken": self._sc}
-        async with self.session as session:
-            async with session.post(
-                "https://customerservice2.southerncompany.com/Account/LoginComplete?"
-                "ReturnUrl=null",
-                data=data,
-            ) as resp:
-                # Checking for unsuccessful login
-                if resp.status != 200:
-                    raise NoScTokenFound(
-                        f"Failed to get secondary ScWebToken: {resp.status} "
-                        f"{resp.headers} {data}"
-                    )
-                # Regex to parse JWT out of headers
-                # NOTE: This used to be ScWebToken before 02/07/2023
-                swtregex = re.compile(r"SouthernJwtCookie=(\S*);", re.IGNORECASE)
-                # Parsing response header to get token
-                swtcookies = resp.headers.get("set-cookie")
-                if swtcookies:
-                    swtmatches = swtregex.search(swtcookies)
+        async with self.session.post(
+            "https://customerservice2.southerncompany.com/Account/LoginComplete?"
+            "ReturnUrl=null",
+            data=data,
+        ) as resp:
+            # Checking for unsuccessful login
+            if resp.status != 200:
+                await self.authenticate()
+                raise NoScTokenFound(
+                    f"Failed to get secondary ScWebToken: {resp.status} "
+                    f"{resp.headers} {data} sc_expiry: {self._sc_expiry}"
+                )
+            # Regex to parse JWT out of headers
+            # NOTE: This used to be ScWebToken before 02/07/2023
+            swtregex = re.compile(r"SouthernJwtCookie=(\S*);", re.IGNORECASE)
+            # Parsing response header to get token
+            swtcookies = resp.headers.get("set-cookie")
+            if swtcookies:
+                swtmatches = swtregex.search(swtcookies)
 
-                    # Checking for matches
-                    if swtmatches and swtmatches.group(1):
-                        swtoken = swtmatches.group(1)
-                    else:
-                        raise NoScTokenFound(
-                            "Failed to get secondary ScWebToken: Could not find any "
-                            "token matches in headers"
-                        )
+                # Checking for matches
+                if swtmatches and swtmatches.group(1):
+                    swtoken = swtmatches.group(1)
                 else:
                     raise NoScTokenFound(
-                        "Failed to get secondary ScWebToken: No cookies were sent back."
+                        "Failed to get secondary ScWebToken: Could not find any "
+                        "token matches in headers"
                     )
+            else:
+                raise NoScTokenFound(
+                    "Failed to get secondary ScWebToken: No cookies were sent back."
+                )
         return swtoken
 
     async def get_jwt(self) -> str:
@@ -181,37 +186,34 @@ class SouthernCompanyAPI:
         # Now fetch JWT after secondary ScWebToken
         # NOTE: This used to be ScWebToken before 02/07/2023
         headers = {"Cookie": f"SouthernJwtCookie={swtoken}"}
-        async with self.session as session:
-            async with session.get(
-                "https://customerservice2.southerncompany.com/Account/LoginValidated/"
-                "JwtToken",
-                headers=headers,
-            ) as resp:
-                if resp.status != 200:
-                    raise NoJwtTokenFound(
-                        f"Failed to get JWT: {resp.status} {await resp.text()} "
-                        f"{headers}"
-                    )
-                # Regex to parse JWT out of headers
-                regex = re.compile(r"ScJwtToken=(\S*);", re.IGNORECASE)
+        async with self.session.get(
+            "https://customerservice2.southerncompany.com/Account/LoginValidated/"
+            "JwtToken",
+            headers=headers,
+        ) as resp:
+            if resp.status != 200:
+                raise NoJwtTokenFound(
+                    f"Failed to get JWT: {resp.status} {await resp.text()} "
+                    f"{headers}"
+                )
+            # Regex to parse JWT out of headers
+            regex = re.compile(r"ScJwtToken=(\S*);", re.IGNORECASE)
 
-                # Parsing response header to get token
-                cookies = resp.headers.get("set-cookie")
-                if cookies:
-                    matches = regex.search(cookies)
+            # Parsing response header to get token
+            cookies = resp.headers.get("set-cookie")
+            if cookies:
+                matches = regex.search(cookies)
 
-                    # Checking for matches
-                    if matches and matches.group(1):
-                        token = matches.group(1)
-                    else:
-                        raise NoJwtTokenFound(
-                            "Failed to get JWT: Could not find any token matches in "
-                            "headers"
-                        )
+                # Checking for matches
+                if matches and matches.group(1):
+                    token = matches.group(1)
                 else:
                     raise NoJwtTokenFound(
-                        "Failed to get JWT: No cookies were sent back."
+                        "Failed to get JWT: Could not find any token matches in "
+                        "headers"
                     )
+            else:
+                raise NoJwtTokenFound("Failed to get JWT: No cookies were sent back.")
 
         # Returning JWT
         self._jwt = token
@@ -224,34 +226,35 @@ class SouthernCompanyAPI:
         if await self.jwt is None:
             raise CantReachSouthernCompany("Can't get jwt. Expired and not refreshed")
         headers = {"Authorization": f"bearer {self._jwt}"}
-        async with self.session as session:
-            async with session.get(
-                "https://customerservice2api.southerncompany.com/api/account/"
-                "getAllAccounts",
-                headers=headers,
-            ) as resp:
-                if resp.status != 200:
-                    raise AccountFailure("failed to get accounts")
+        async with self.session.get(
+            "https://customerservice2api.southerncompany.com/api/account/"
+            "getAllAccounts",
+            headers=headers,
+        ) as resp:
+            if resp.status != 200:
+                raise AccountFailure("failed to get accounts")
+            try:
+                account_json = await resp.json()
+            except (ContentTypeError, json.JSONDecodeError) as err:
                 try:
-                    account_json = await resp.json()
-                except (ContentTypeError, json.JSONDecodeError) as err:
-                    try:
-                        error_text = await resp.text()
-                    except aiohttp.ClientError:
-                        error_text = err.msg
-                    raise CantReachSouthernCompany(
-                        f"Incorrect mimetype while trying to get accounts. {error_text}"
-                    ) from err
-                accounts = []
-                for account in account_json["Data"]:
-                    accounts.append(
-                        Account(
-                            name=account["Description"],
-                            primary=account["PrimaryAccount"] == "Y",
-                            number=account["AccountNumber"],
-                            company=COMPANY_MAP.get(account["Company"], Company.GPC),
-                            session=self.session,
-                        )
+                    error_text = await resp.text()
+                except aiohttp.ClientError:
+                    error_text = err.msg
+                raise CantReachSouthernCompany(
+                    f"Incorrect mimetype while trying to get accounts. {error_text}"
+                ) from err
+            accounts = []
+            for account in account_json["Data"]:
+                accounts.append(
+                    Account(
+                        name=account["Description"],
+                        primary=account["PrimaryAccount"] == "Y",
+                        number=account["AccountNumber"],
+                        company=COMPANY_MAP.get(account["Company"], Company.GPC),
+                        session=self.session,
                     )
+                )
+        for account in accounts:
+            await account.get_service_point_number(self._jwt)
         self._accounts = accounts
         return accounts
